{-# LANGUAGE FlexibleContexts, RankNTypes, ConstraintKinds #-}

{-
  This module is part of Ironforge.
  Copyfree (f) 2014 Marvin Cohrs

  All wrongs revoked. Sharing is an act of love, not crime.
  Please share Ironforge with everyone you like.
  
  Redistribution and use in source and binary forms, with or without modification,
  are permitted provided that the following conditions are met:

      * Redistributions of source code must retain the above copyright notice,
        this list of conditions and the following disclaimer.
      * Redistributions in binary form must reproduce the above copyright notice,
        this list of conditions and the following disclaimer in the documentation
        and/or other materials provided with the distribution.
      * Neither the name of Marvin Cohrs nor the names of other
        contributors may be used to endorse or promote products derived
        from this software without specific prior written permission.
  
  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-}

module Game.Antisplice.Dungeon.Ironforge.Kinds where

import Text.Chatty.Printer
import Text.Chatty.Scanner
import Text.Chatty.Expansion
import Text.Chatty.Expansion.Vars
import System.Chatty.Misc
import Text.Chatty.Extended.Printer
import Game.Antisplice
import Data.Chatty.Graph
import Control.Monad
import Control.Monad.Error.Class
import Data.Monoid
import Text.Printf

data Kinds = Kinds {
    kOfficer :: KindId,
    kCabbage :: KindId,
    kCarrot :: KindId,
    kCereliac :: KindId,
    kLeek :: KindId,
    kBeetroot :: KindId,
    kPotato :: KindId,
    kApple :: KindId,
    kBanana :: KindId,
    kWater :: KindId,
    kBeer :: KindId,
    kStrongBeer :: KindId,
    kAppleJuice :: KindId,
    kVodka :: KindId,
    kOrangeLemonade :: KindId,
    kPancake :: KindId,
    kAppleSauce :: KindId,
    kFire :: KindId,
    kFlour :: KindId,
    kEgg :: KindId,
    kMilk :: KindId,
    kSoda :: KindId,
    kSugar :: KindId
  }

data Currencies = Currencies {
    money :: CurrencyId,
    mana :: CurrencyId
  }

data Methods = Methods {
    byCooking :: RecipeMethod,
    byBuilding :: RecipeMethod
  }

data Atoms = Atoms {
    washed :: Atom Bool
  }

getKinds :: Constructor Kinds
getKinds = do
  [kOfficer, kCabbage, kCarrot, kCereliac, kLeek,
   kBeetroot, kPotato, kApple, kBanana, kWater,
   kBeer, kStrongBeer, kAppleJuice, kVodka,
   kOrangeLemonade, kPancake, kAppleSauce,
   kFire, kFlour, kEgg, kMilk, kSoda,
   kSugar] <- forM [1..23] $ const registerKind
  return $ Kinds
    kOfficer kCabbage kCarrot kCereliac kLeek
    kBeetroot kPotato kApple kBanana kWater
    kBeer kStrongBeer kAppleJuice kVodka
    kOrangeLemonade kPancake kAppleSauce
    kFire kFlour kEgg kMilk kSoda kSugar

getCurrencies :: Constructor Currencies
getCurrencies = do
  money <- registerCurrency "money" "Money is measured in Iron Dollars. You can pay with this."
  mana <- registerCurrency "mana" "Mana is used to cast spells."
  return $ Currencies
    money mana

getMethods :: Constructor Methods
getMethods = do
  byCooking <- liftM RecipeMethod countOn
  byBuilding <- liftM RecipeMethod countOn
  return $ Methods
    byCooking byBuilding

getAtoms :: Constructor Atoms
getAtoms = do
  washed <- newAtom
  return $ Atoms
    washed
